@@ -16,17 +16,17 @@
 import copy
 import datetime
 import os
-import types
-import urlparse
 import uuid
 
 import eventlet.event
 from oslo_config import cfg
 from oslo_log import log as logging
+from oslo_serialization import base64
+import six
 from yaql import specs
 
 import murano.common.exceptions as exceptions
-import murano.common.messaging as messaging
+from murano.common.messaging import message
 from murano.dsl import dsl
 import murano.engine.system.common as common
 
@@ -40,35 +40,36 @@ class AgentException(Exception):
 
 @dsl.name('io.murano.system.Agent')
 class Agent(object):
-    def __init__(self, interfaces, host):
+    def __init__(self, host):
         self._enabled = False
         if CONF.engine.disable_murano_agent:
             LOG.debug('Use of murano-agent is disallowed '
                       'by the server configuration')
             return
 
-        self._environment = self._get_environment(interfaces, host)
+        self._host = host
         self._enabled = True
-        self._queue = str('e%s-h%s' % (
-            self._environment.id, host.id)).lower()
-
-    def _get_environment(self, interfaces, host):
-        return interfaces.yaql()(
-            "$.find('io.murano.Environment').require()", host)
+        env = host.find_owner('io.murano.Environment')
+        self._queue = str('e%s-h%s' % (env.id, host.id)).lower()
+        self._initialized = False
 
     @property
     def enabled(self):
         return self._enabled
 
-    def prepare(self):
+    def _initialize(self):
+        if self._initialized:
+            return
+
         # (sjmc7) - turn this into a no-op if agents are disabled
         if CONF.engine.disable_murano_agent:
             LOG.debug('Use of murano-agent is disallowed '
                       'by the server configuration')
-            return
-
-        with common.create_rmq_client() as client:
-            client.declare(self._queue, enable_ha=True, ttl=86400000)
+        else:
+            region = dsl.MuranoObjectInterface.create(self._host().getRegion())
+            with common.create_rmq_client(region) as client:
+                client.declare(self._queue, enable_ha=True, ttl=86400000)
+        self._initialized = True
 
     def queue_name(self):
         return self._queue
@@ -80,21 +81,23 @@ class Agent(object):
                 'by the server configuration')
 
     def _prepare_message(self, template, msg_id):
-        msg = messaging.Message()
+        msg = message.Message()
         msg.body = template
         msg.id = msg_id
         return msg
 
     def _send(self, template, wait_results, timeout):
         """Send a message over the MQ interface."""
+        self._initialize()
+        region = self._host().getRegion()
         msg_id = template.get('ID', uuid.uuid4().hex)
         if wait_results:
             event = eventlet.event.Event()
-            listener = self._environment['agentListener']
+            listener = region['agentListener']
             listener().subscribe(msg_id, event)
 
         msg = self._prepare_message(template, msg_id)
-        with common.create_rmq_client() as client:
+        with common.create_rmq_client(region) as client:
             client.send(message=msg, key=self._queue)
 
         if wait_results:
@@ -121,7 +124,7 @@ class Agent(object):
             return None
 
     @specs.parameter(
-        'resources', dsl.MuranoType('io.murano.system.Resources'))
+        'resources', dsl.MuranoObjectParameter('io.murano.system.Resources'))
     def call(self, template, resources, timeout=None):
         if timeout is None:
             timeout = CONF.engine.agent_timeout
@@ -130,7 +133,7 @@ class Agent(object):
         return self._send(plan, True, timeout)
 
     @specs.parameter(
-        'resources', dsl.MuranoType('io.murano.system.Resources'))
+        'resources', dsl.MuranoObjectParameter('io.murano.system.Resources'))
     def send(self, template, resources):
         self._check_enabled()
         plan = self.build_execution_plan(template, resources())
@@ -157,7 +160,7 @@ class Agent(object):
     def wait_ready(self, timeout=100):
         self._check_enabled()
         template = {'Body': 'return', 'FormatVersion': '2.0.0', 'Scripts': {}}
-        self.call(template, False, timeout)
+        self.call_raw(template, timeout)
 
     def _process_v1_result(self, result):
         if result['IsException']:
@@ -207,7 +210,7 @@ class Agent(object):
 
     def build_execution_plan(self, template, resources):
         template = copy.deepcopy(template)
-        if not isinstance(template, types.DictionaryType):
+        if not isinstance(template, dict):
             raise ValueError('Incorrect execution plan ')
         format_version = template.get('FormatVersion')
         if not format_version or format_version.startswith('1.'):
@@ -221,8 +224,8 @@ class Agent(object):
         scripts = []
         for script in script_files:
             script_path = os.path.join(scripts_folder, script)
-            scripts.append(resources.string(
-                script_path).encode('base64'))
+            script_path = base64.encode_as_text(script_path) + "\n"
+            scripts.append(resources.string(script_path, binary=True))
         template['Scripts'] = scripts
         return template
 
@@ -261,7 +264,7 @@ class Agent(object):
 
     def _is_url(self, file):
         file = self._get_url(file)
-        parts = urlparse.urlsplit(file)
+        parts = six.moves.urllib.parse.urlsplit(file)
         if not parts.scheme or not parts.netloc:
             return False
         else:
@@ -269,13 +272,13 @@ class Agent(object):
 
     def _get_url(self, file):
         if isinstance(file, dict):
-            return file.values()[0]
+            return list(file.values())[0]
         else:
             return file
 
     def _get_name(self, file):
         if isinstance(file, dict):
-            name = file.keys()[0]
+            name = list(file.keys())[0]
         else:
             name = file
 
@@ -287,16 +290,18 @@ class Agent(object):
 
     def _get_file_value(self, file):
         if isinstance(file, dict):
-            file = file.values()[0]
+            file = list(file.values())[0]
         return file
 
     def _get_body(self, file, resources, folder):
         use_base64 = self._is_base64(file)
-        if use_base64 and file.startswith('<') and file.endswith('>'):
-            file = file[1: -1]
-        body = resources.string(os.path.join(folder, file))
         if use_base64:
-            body = body.encode('base64')
+            path = os.path.join(folder, file[1: -1])
+            body = resources.string(path, binary=True)
+            body = base64.encode_as_text(body) + "\n"
+        else:
+            path = os.path.join(folder, file)
+            body = resources.string(path)
         return body
 
     def _is_base64(self, file):
